#!/usr/bin/env bash

set -e

put_nginx_config() {
    cat <<EOF > /etc/nginx/nginx.conf
user ${NGINX_USER};
events {}

http {
    server {
        listen 8081;
        location /deb/ {
            proxy_ssl_verify on;
            proxy_ssl_trusted_certificate ${CERT_PATH}/mcc.crt;
            proxy_pass ${ITEM_PARAM_MCP_APT_MIRROR}/ ;
        }
        location /bin/ {
            proxy_ssl_verify on;
            proxy_ssl_trusted_certificate ${CERT_PATH}/mcc.crt;
            proxy_pass ${ITEM_PARAM_MCP_BINARY_REGISTRY}/ ;
        }
    }
}
EOF
}

apt_get_update() {
    retries=20
    retry=1
    while apt-get update | tee /dev/stderr | grep -E "^Err:"
    do
        if [[ $retry -le $retries ]]; then
            echo "apt-get update command has failed. Retrying ${retry}/${retries}"
            retry=$((retry+1))
            sleep 5
        else
            echo "The command has failed after ${retries} attempts" && exit 1
        fi
    done
}

source vars.sh

if [ -e /etc/system-release ]; then
    OS_FAMILY="rhel"
else
    OS_FAMILY="debian"
fi

: "${ITEM_PARAM_MCP_APT_MIRROR:="https://mirror.mirantis.com"}"
: "${ITEM_PARAM_MCP_BINARY_REGISTRY:="https://binary.mirantis.com"}"
: "${ITEM_PARAM_PROXY_ENABLED}:=${PROXY_ENABLED}"

if ! timeout 1 bash -c 'cat < /dev/null > /dev/tcp/127.0.0.1/8081' &> /dev/null && [ "${ITEM_PARAM_PROXY_ENABLED}" == "true" ]; then
    # proxy enabled
    if [ "$OS_FAMILY" == "debian" ]; then
        : "${ITEM_PARAM_KAAS_UBUNTU_REPO:=${KAAS_UBUNTU_REPO}}"
        : "${ITEM_PARAM_KAAS_UBUNTU_CODENAME:=${KAAS_UBUNTU_CODENAME}}"

        export DEBIAN_FRONTEND=noninteractive
        export DEBCONF_NONINTERACTIVE_SEEN=true
        APT_OPTS="-o APT::Install-Suggests=0 -o APT::Install-Recommends=0"
        NGINX_USER="www-data"
        CERT_PATH="/usr/local/share/ca-certificates"

        # Remove default repositories
        find /etc/apt/ -name *.list -delete

        # Add Ubuntu mirrors
        cat <<EOF > /etc/apt/sources.list
# Ubuntu snapshot
deb [arch=amd64] ${ITEM_PARAM_MCP_APT_MIRROR}/${ITEM_PARAM_KAAS_UBUNTU_REPO} ${ITEM_PARAM_KAAS_UBUNTU_CODENAME} main restricted universe
deb [arch=amd64] ${ITEM_PARAM_MCP_APT_MIRROR}/${ITEM_PARAM_KAAS_UBUNTU_REPO} ${ITEM_PARAM_KAAS_UBUNTU_CODENAME}-updates main restricted universe
deb [arch=amd64] ${ITEM_PARAM_MCP_APT_MIRROR}/${ITEM_PARAM_KAAS_UBUNTU_REPO} ${ITEM_PARAM_KAAS_UBUNTU_CODENAME}-security main restricted universe
EOF

        # Install nginx
        apt_get_update
        apt-get install -y ${APT_OPTS} nginx
        put_nginx_config
        systemctl restart nginx
    else
        NGINX_USER="nginx"
        CERT_PATH="/usr/share/pki/ca-trust-source/anchors"

        source <(cat /etc/os-release | grep VERSION_ID)
        VERSION_ID="${VERSION_ID%%.*}"
        source <(cat /etc/os-release | grep -E "^ID=")
        if [ "${ID}" == "rhel" ]; then
            yum -y install yum-utils
            yum-config-manager --enable \
            rhui-rhel-${VERSION_ID}-server-rhui-extras-rpms \
            rhel-${VERSION_ID}-server-extras-rpms
        fi
        cat <<EOF > /etc/yum.repos.d/kubernetes_extra.repo
[kubernetes_extra]
name=Kubernetes Extra
baseurl=${ITEM_PARAM_MCP_APT_MIRROR}/kaas/kubernetes-extra-rhel${VERSION_ID}-0.0.2/
gpgcheck=0
enabled=1
EOF
        yum -y install nginx firewalld
        systemctl start firewalld
        firewall-cmd --permanent --add-port=8081/tcp
        semanage port -m -t http_port_t -p tcp 8081
        put_nginx_config
        systemctl restart nginx
    fi
fi
