from ansible.module_utils.basic import AnsibleModule
import codecs
import ipaddress
import sys
try:
    unicode = unicode
except NameError:
    unicode = str

def decodeIP(s):
    ss = ""
    for x in range(0, len(s), 2):
        ss = "{}{}".format(s[x:x+2], ss)
    return ipaddress.IPv4Address(codecs.decode(ss, 'hex_codec'))

def decodeIP2string(s):
    return "{}".format(decodeIP(s))

def getLocalRouteTable():
    routefile = '/proc/net/route'
    rv = []
    with open(routefile) as f:
        for l in f:
            fields=l.split()
            if fields[2] != "00000000": # only local-links interested us
                continue
            rv.append((
                fields[0],
                ipaddress.IPv4Network(unicode(
                    "{}/{}".format(decodeIP2string(fields[1]), decodeIP2string(fields[7]))),
                    strict=False
                )
            ))
    return rv

def main():
    module = AnsibleModule(
        argument_spec=dict(
            lb_ipaddr=dict(required=True, default=None),
        ),
        supports_check_mode=True
    )

    rv = "not-found"
    lbIp = ipaddress.IPv4Address(unicode(module.params['lb_ipaddr']))

    # error-based presumption:
    changed = False
    failed = True
    rc = 254

    routeTable = getLocalRouteTable()
    for rou in routeTable:
        if lbIp in rou[1].hosts():
            rv = rou[0]
            break

    if rc != "not-found":
        # All right, result found :)
        changed = True
        failed = False
        rc = 0

    result = dict(
        iface=rv,
        changed=changed,
        failed=failed,
        rc=rc
    )
    module.exit_json(**result)

if __name__ == '__main__':
    main()
