#!/usr/bin/env bash

set -e

apt_get_update() {
    retries=20
    retry=1
    while apt-get update | tee /dev/stderr | grep -E "^Err:"
    do
        if [[ $retry -le $retries ]]; then
            echo "apt-get update command has failed. Retrying ${retry}/${retries}"
            retry=$((retry+1))
            sleep 5
        else
            echo "The command has failed after ${retries} attempts" && exit 1
        fi
    done
}

source vars.sh

if [ -e /etc/system-release ]; then
    OS_FAMILY="rhel"
else
    OS_FAMILY="debian"
fi

: "${ITEM_PARAM_MCP_APT_MIRROR:="https://mirror.mirantis.com"}"

if [ "$OS_FAMILY" == "debian" ]; then
    : "${ITEM_PARAM_KAAS_UBUNTU_REPO:=${KAAS_UBUNTU_REPO}}"
    : "${ITEM_PARAM_KAAS_UBUNTU_CODENAME:=${KAAS_UBUNTU_CODENAME}}"
    K8S_EXTRA_REPO_VERSION='0.0.7'
    K8S_EXTRA_REPO_KEY="archive-kubernetes-extra-${K8S_EXTRA_REPO_VERSION}.key"
    K8S_EXTRA_REPO_URL="${ITEM_PARAM_MCP_APT_MIRROR}/kaas/kubernetes-extra-${K8S_EXTRA_REPO_VERSION}/${ITEM_PARAM_KAAS_UBUNTU_CODENAME}"

    export DEBIAN_FRONTEND=noninteractive
    export DEBCONF_NONINTERACTIVE_SEEN=true
    APT_OPTS="-o APT::Install-Suggests=0 -o APT::Install-Recommends=0"
    # Install ca-certificates to access Mirantis mirror
    apt_get_update
    dpkg -s ca-certificates &> /dev/null || apt-get install -y ${APT_OPTS} ca-certificates

    # Remove default repositories
    find /etc/apt/ -name *.list -delete

    # Add Ubuntu mirrors
    cat <<EOF > /etc/apt/sources.list
# Ubuntu snapshot
deb [arch=amd64] ${ITEM_PARAM_MCP_APT_MIRROR}/${ITEM_PARAM_KAAS_UBUNTU_REPO} ${ITEM_PARAM_KAAS_UBUNTU_CODENAME} main restricted universe
deb [arch=amd64] ${ITEM_PARAM_MCP_APT_MIRROR}/${ITEM_PARAM_KAAS_UBUNTU_REPO} ${ITEM_PARAM_KAAS_UBUNTU_CODENAME}-updates main restricted universe
deb [arch=amd64] ${ITEM_PARAM_MCP_APT_MIRROR}/${ITEM_PARAM_KAAS_UBUNTU_REPO} ${ITEM_PARAM_KAAS_UBUNTU_CODENAME}-security main restricted universe
EOF

    # Prefer Mirantis owned packages over Ubuntu packages
    cat <<EOF > /etc/apt/preferences.d/mirantis
Package: *
Pin: release o=Mirantis
Pin-Priority: 1001
EOF

    # Install package prerequisites
    apt_get_update
    apt-get install -y ${APT_OPTS} curl gnupg-agent

    update-alternatives --install /usr/bin/python python /usr/bin/python3 1

    # Add kubernetes-extra repo and install Ansible
    cat <<EOF >> /etc/apt/sources.list
    # kubernetes-extra repo
deb ${K8S_EXTRA_REPO_URL} ${ITEM_PARAM_KAAS_UBUNTU_CODENAME} main
EOF

    curl -fsSL "${K8S_EXTRA_REPO_URL}/${K8S_EXTRA_REPO_KEY}" | apt-key add -
    apt_get_update
    apt-get install -y ${APT_OPTS} ansible

else
    source <(cat /etc/os-release | grep VERSION_ID)
    VERSION_ID="${VERSION_ID%%.*}"
    source <(cat /etc/os-release | grep -E "^ID=")
    if [ "${ID}" == "rhel" ]; then
        yum -y install yum-utils
        yum-config-manager --enable \
        rhui-rhel-${VERSION_ID}-server-rhui-extras-rpms \
        rhel-${VERSION_ID}-server-extras-rpms
    fi
    yum -y install ca-certificates curl
    cat <<EOF > /etc/yum.repos.d/kubernetes_extra.repo
[kubernetes_extra]
name=Kubernetes Extra
baseurl=${ITEM_PARAM_MCP_APT_MIRROR}/kaas/kubernetes-extra-rhel${VERSION_ID}-0.0.2/
gpgcheck=0
enabled=1
EOF
    yum -y install ansible
fi

ansible-playbook --version
